# Generated from CharmmCRDParser.g4 by ANTLR 4.13.0
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO

def serializedATN():
    return [
        4,1,14,45,2,0,7,0,2,1,7,1,2,2,7,2,2,3,7,3,1,0,1,0,5,0,11,8,0,10,
        0,12,0,14,9,0,1,0,1,0,1,1,1,1,5,1,20,8,1,10,1,12,1,23,9,1,1,1,1,
        1,1,2,1,2,1,2,4,2,30,8,2,11,2,12,2,31,1,3,1,3,1,3,1,3,1,3,1,3,1,
        3,1,3,1,3,1,3,1,3,1,3,0,0,4,0,2,4,6,0,1,1,1,14,14,44,0,12,1,0,0,
        0,2,17,1,0,0,0,4,26,1,0,0,0,6,33,1,0,0,0,8,11,3,2,1,0,9,11,3,4,2,
        0,10,8,1,0,0,0,10,9,1,0,0,0,11,14,1,0,0,0,12,10,1,0,0,0,12,13,1,
        0,0,0,13,15,1,0,0,0,14,12,1,0,0,0,15,16,5,0,0,1,16,1,1,0,0,0,17,
        21,5,4,0,0,18,20,5,12,0,0,19,18,1,0,0,0,20,23,1,0,0,0,21,19,1,0,
        0,0,21,22,1,0,0,0,22,24,1,0,0,0,23,21,1,0,0,0,24,25,7,0,0,0,25,3,
        1,0,0,0,26,27,5,1,0,0,27,29,5,5,0,0,28,30,3,6,3,0,29,28,1,0,0,0,
        30,31,1,0,0,0,31,29,1,0,0,0,31,32,1,0,0,0,32,5,1,0,0,0,33,34,5,1,
        0,0,34,35,5,1,0,0,35,36,5,6,0,0,36,37,5,6,0,0,37,38,5,2,0,0,38,39,
        5,2,0,0,39,40,5,2,0,0,40,41,5,6,0,0,41,42,5,1,0,0,42,43,5,2,0,0,
        43,7,1,0,0,0,4,10,12,21,31
    ]

class CharmmCRDParser ( Parser ):

    grammarFileName = "CharmmCRDParser.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "'EXT'" ]

    symbolicNames = [ "<INVALID>", "Integer", "Float", "Double_quote_string", 
                      "COMMENT", "Ext", "Simple_name", "SPACE", "CONTINUE", 
                      "ENCLOSE_COMMENT", "SECTION_COMMENT", "LINE_COMMENT", 
                      "Any_name", "SPACE_CM", "RETURN_CM" ]

    RULE_charmm_crd = 0
    RULE_comment = 1
    RULE_coordinates = 2
    RULE_atom_coordinate = 3

    ruleNames =  [ "charmm_crd", "comment", "coordinates", "atom_coordinate" ]

    EOF = Token.EOF
    Integer=1
    Float=2
    Double_quote_string=3
    COMMENT=4
    Ext=5
    Simple_name=6
    SPACE=7
    CONTINUE=8
    ENCLOSE_COMMENT=9
    SECTION_COMMENT=10
    LINE_COMMENT=11
    Any_name=12
    SPACE_CM=13
    RETURN_CM=14

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.13.0")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class Charmm_crdContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EOF(self):
            return self.getToken(CharmmCRDParser.EOF, 0)

        def comment(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CharmmCRDParser.CommentContext)
            else:
                return self.getTypedRuleContext(CharmmCRDParser.CommentContext,i)


        def coordinates(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CharmmCRDParser.CoordinatesContext)
            else:
                return self.getTypedRuleContext(CharmmCRDParser.CoordinatesContext,i)


        def getRuleIndex(self):
            return CharmmCRDParser.RULE_charmm_crd

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCharmm_crd" ):
                listener.enterCharmm_crd(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCharmm_crd" ):
                listener.exitCharmm_crd(self)




    def charmm_crd(self):

        localctx = CharmmCRDParser.Charmm_crdContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_charmm_crd)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 12
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==1 or _la==4:
                self.state = 10
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [4]:
                    self.state = 8
                    self.comment()
                    pass
                elif token in [1]:
                    self.state = 9
                    self.coordinates()
                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 14
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 15
            self.match(CharmmCRDParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class CommentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def COMMENT(self):
            return self.getToken(CharmmCRDParser.COMMENT, 0)

        def RETURN_CM(self):
            return self.getToken(CharmmCRDParser.RETURN_CM, 0)

        def EOF(self):
            return self.getToken(CharmmCRDParser.EOF, 0)

        def Any_name(self, i:int=None):
            if i is None:
                return self.getTokens(CharmmCRDParser.Any_name)
            else:
                return self.getToken(CharmmCRDParser.Any_name, i)

        def getRuleIndex(self):
            return CharmmCRDParser.RULE_comment

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterComment" ):
                listener.enterComment(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitComment" ):
                listener.exitComment(self)




    def comment(self):

        localctx = CharmmCRDParser.CommentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_comment)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 17
            self.match(CharmmCRDParser.COMMENT)
            self.state = 21
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==12:
                self.state = 18
                self.match(CharmmCRDParser.Any_name)
                self.state = 23
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 24
            _la = self._input.LA(1)
            if not(_la==-1 or _la==14):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class CoordinatesContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self):
            return self.getToken(CharmmCRDParser.Integer, 0)

        def Ext(self):
            return self.getToken(CharmmCRDParser.Ext, 0)

        def atom_coordinate(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CharmmCRDParser.Atom_coordinateContext)
            else:
                return self.getTypedRuleContext(CharmmCRDParser.Atom_coordinateContext,i)


        def getRuleIndex(self):
            return CharmmCRDParser.RULE_coordinates

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCoordinates" ):
                listener.enterCoordinates(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCoordinates" ):
                listener.exitCoordinates(self)




    def coordinates(self):

        localctx = CharmmCRDParser.CoordinatesContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_coordinates)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 26
            self.match(CharmmCRDParser.Integer)
            self.state = 27
            self.match(CharmmCRDParser.Ext)
            self.state = 29 
            self._errHandler.sync(self)
            _alt = 1
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt == 1:
                    self.state = 28
                    self.atom_coordinate()

                else:
                    raise NoViableAltException(self)
                self.state = 31 
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,3,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Atom_coordinateContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CharmmCRDParser.Integer)
            else:
                return self.getToken(CharmmCRDParser.Integer, i)

        def Simple_name(self, i:int=None):
            if i is None:
                return self.getTokens(CharmmCRDParser.Simple_name)
            else:
                return self.getToken(CharmmCRDParser.Simple_name, i)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(CharmmCRDParser.Float)
            else:
                return self.getToken(CharmmCRDParser.Float, i)

        def getRuleIndex(self):
            return CharmmCRDParser.RULE_atom_coordinate

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAtom_coordinate" ):
                listener.enterAtom_coordinate(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAtom_coordinate" ):
                listener.exitAtom_coordinate(self)




    def atom_coordinate(self):

        localctx = CharmmCRDParser.Atom_coordinateContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_atom_coordinate)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 33
            self.match(CharmmCRDParser.Integer)
            self.state = 34
            self.match(CharmmCRDParser.Integer)
            self.state = 35
            self.match(CharmmCRDParser.Simple_name)
            self.state = 36
            self.match(CharmmCRDParser.Simple_name)
            self.state = 37
            self.match(CharmmCRDParser.Float)
            self.state = 38
            self.match(CharmmCRDParser.Float)
            self.state = 39
            self.match(CharmmCRDParser.Float)
            self.state = 40
            self.match(CharmmCRDParser.Simple_name)
            self.state = 41
            self.match(CharmmCRDParser.Integer)
            self.state = 42
            self.match(CharmmCRDParser.Float)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx








# Generated from VnmrPKParser.g4 by ANTLR 4.13.0
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO

def serializedATN():
    return [
        4,1,52,236,2,0,7,0,2,1,7,1,2,2,7,2,2,3,7,3,2,4,7,4,2,5,7,5,2,6,7,
        6,2,7,7,7,2,8,7,8,2,9,7,9,2,10,7,10,1,0,3,0,24,8,0,1,0,1,0,1,0,1,
        0,4,0,30,8,0,11,0,12,0,31,1,0,4,0,35,8,0,11,0,12,0,36,1,0,4,0,40,
        8,0,11,0,12,0,41,5,0,44,8,0,10,0,12,0,47,9,0,1,0,1,0,1,1,1,1,5,1,
        53,8,1,10,1,12,1,56,9,1,1,1,1,1,1,2,1,2,1,2,1,2,1,2,3,2,65,8,2,1,
        2,3,2,68,8,2,1,2,1,2,3,2,72,8,2,1,2,3,2,75,8,2,1,2,3,2,78,8,2,1,
        2,3,2,81,8,2,1,2,3,2,84,8,2,1,2,3,2,87,8,2,1,2,1,2,4,2,91,8,2,11,
        2,12,2,92,1,2,4,2,96,8,2,11,2,12,2,97,1,2,4,2,101,8,2,11,2,12,2,
        102,3,2,105,8,2,1,3,1,3,1,3,1,3,1,3,3,3,112,8,3,1,3,3,3,115,8,3,
        1,3,3,3,118,8,3,1,3,3,3,121,8,3,1,3,1,3,1,4,1,4,1,4,1,4,1,4,1,4,
        3,4,131,8,4,1,4,3,4,134,8,4,1,4,3,4,137,8,4,1,4,3,4,140,8,4,1,4,
        3,4,143,8,4,1,4,1,4,1,5,1,5,1,5,1,5,1,5,1,5,1,5,3,5,154,8,5,1,5,
        3,5,157,8,5,1,5,3,5,160,8,5,1,5,3,5,163,8,5,1,5,3,5,166,8,5,1,5,
        3,5,169,8,5,1,5,1,5,1,6,1,6,1,6,1,6,1,6,1,6,1,6,1,6,1,6,3,6,182,
        8,6,3,6,184,8,6,1,6,1,6,3,6,188,8,6,1,6,3,6,191,8,6,1,6,1,6,1,7,
        1,7,1,7,1,7,1,7,1,7,1,7,3,7,202,8,7,1,7,1,7,1,8,1,8,1,8,1,8,1,8,
        1,8,1,8,1,8,1,8,3,8,215,8,8,1,8,1,8,1,9,1,9,1,9,1,9,1,9,1,9,1,9,
        1,9,1,9,1,9,1,9,3,9,230,8,9,1,9,1,9,1,10,1,10,1,10,0,0,11,0,2,4,
        6,8,10,12,14,16,18,20,0,7,1,1,52,52,2,0,39,39,43,43,2,0,40,40,44,
        44,2,0,41,41,45,45,2,0,42,42,46,46,1,0,26,27,1,0,3,5,270,0,23,1,
        0,0,0,2,50,1,0,0,0,4,59,1,0,0,0,6,106,1,0,0,0,8,124,1,0,0,0,10,146,
        1,0,0,0,12,172,1,0,0,0,14,194,1,0,0,0,16,205,1,0,0,0,18,218,1,0,
        0,0,20,233,1,0,0,0,22,24,5,14,0,0,23,22,1,0,0,0,23,24,1,0,0,0,24,
        45,1,0,0,0,25,44,3,2,1,0,26,44,3,4,2,0,27,44,3,12,6,0,28,30,3,14,
        7,0,29,28,1,0,0,0,30,31,1,0,0,0,31,29,1,0,0,0,31,32,1,0,0,0,32,44,
        1,0,0,0,33,35,3,16,8,0,34,33,1,0,0,0,35,36,1,0,0,0,36,34,1,0,0,0,
        36,37,1,0,0,0,37,44,1,0,0,0,38,40,3,18,9,0,39,38,1,0,0,0,40,41,1,
        0,0,0,41,39,1,0,0,0,41,42,1,0,0,0,42,44,1,0,0,0,43,25,1,0,0,0,43,
        26,1,0,0,0,43,27,1,0,0,0,43,29,1,0,0,0,43,34,1,0,0,0,43,39,1,0,0,
        0,44,47,1,0,0,0,45,43,1,0,0,0,45,46,1,0,0,0,46,48,1,0,0,0,47,45,
        1,0,0,0,48,49,5,0,0,1,49,1,1,0,0,0,50,54,5,6,0,0,51,53,5,50,0,0,
        52,51,1,0,0,0,53,56,1,0,0,0,54,52,1,0,0,0,54,55,1,0,0,0,55,57,1,
        0,0,0,56,54,1,0,0,0,57,58,7,0,0,0,58,3,1,0,0,0,59,60,5,2,0,0,60,
        61,5,32,0,0,61,62,5,33,0,0,62,64,5,34,0,0,63,65,5,35,0,0,64,63,1,
        0,0,0,64,65,1,0,0,0,65,67,1,0,0,0,66,68,5,36,0,0,67,66,1,0,0,0,67,
        68,1,0,0,0,68,69,1,0,0,0,69,71,5,37,0,0,70,72,5,38,0,0,71,70,1,0,
        0,0,71,72,1,0,0,0,72,74,1,0,0,0,73,75,7,1,0,0,74,73,1,0,0,0,74,75,
        1,0,0,0,75,77,1,0,0,0,76,78,7,2,0,0,77,76,1,0,0,0,77,78,1,0,0,0,
        78,80,1,0,0,0,79,81,7,3,0,0,80,79,1,0,0,0,80,81,1,0,0,0,81,83,1,
        0,0,0,82,84,7,4,0,0,83,82,1,0,0,0,83,84,1,0,0,0,84,86,1,0,0,0,85,
        87,5,47,0,0,86,85,1,0,0,0,86,87,1,0,0,0,87,88,1,0,0,0,88,104,5,49,
        0,0,89,91,3,6,3,0,90,89,1,0,0,0,91,92,1,0,0,0,92,90,1,0,0,0,92,93,
        1,0,0,0,93,105,1,0,0,0,94,96,3,8,4,0,95,94,1,0,0,0,96,97,1,0,0,0,
        97,95,1,0,0,0,97,98,1,0,0,0,98,105,1,0,0,0,99,101,3,10,5,0,100,99,
        1,0,0,0,101,102,1,0,0,0,102,100,1,0,0,0,102,103,1,0,0,0,103,105,
        1,0,0,0,104,90,1,0,0,0,104,95,1,0,0,0,104,100,1,0,0,0,105,5,1,0,
        0,0,106,107,5,3,0,0,107,108,5,4,0,0,108,109,5,4,0,0,109,111,3,20,
        10,0,110,112,3,20,10,0,111,110,1,0,0,0,111,112,1,0,0,0,112,114,1,
        0,0,0,113,115,3,20,10,0,114,113,1,0,0,0,114,115,1,0,0,0,115,117,
        1,0,0,0,116,118,3,20,10,0,117,116,1,0,0,0,117,118,1,0,0,0,118,120,
        1,0,0,0,119,121,5,7,0,0,120,119,1,0,0,0,120,121,1,0,0,0,121,122,
        1,0,0,0,122,123,5,14,0,0,123,7,1,0,0,0,124,125,5,3,0,0,125,126,5,
        4,0,0,126,127,5,4,0,0,127,128,5,4,0,0,128,130,3,20,10,0,129,131,
        3,20,10,0,130,129,1,0,0,0,130,131,1,0,0,0,131,133,1,0,0,0,132,134,
        3,20,10,0,133,132,1,0,0,0,133,134,1,0,0,0,134,136,1,0,0,0,135,137,
        3,20,10,0,136,135,1,0,0,0,136,137,1,0,0,0,137,139,1,0,0,0,138,140,
        3,20,10,0,139,138,1,0,0,0,139,140,1,0,0,0,140,142,1,0,0,0,141,143,
        5,7,0,0,142,141,1,0,0,0,142,143,1,0,0,0,143,144,1,0,0,0,144,145,
        5,14,0,0,145,9,1,0,0,0,146,147,5,3,0,0,147,148,5,4,0,0,148,149,5,
        4,0,0,149,150,5,4,0,0,150,151,5,4,0,0,151,153,3,20,10,0,152,154,
        3,20,10,0,153,152,1,0,0,0,153,154,1,0,0,0,154,156,1,0,0,0,155,157,
        3,20,10,0,156,155,1,0,0,0,156,157,1,0,0,0,157,159,1,0,0,0,158,160,
        3,20,10,0,159,158,1,0,0,0,159,160,1,0,0,0,160,162,1,0,0,0,161,163,
        3,20,10,0,162,161,1,0,0,0,162,163,1,0,0,0,163,165,1,0,0,0,164,166,
        3,20,10,0,165,164,1,0,0,0,165,166,1,0,0,0,166,168,1,0,0,0,167,169,
        5,7,0,0,168,167,1,0,0,0,168,169,1,0,0,0,169,170,1,0,0,0,170,171,
        5,14,0,0,171,11,1,0,0,0,172,173,5,1,0,0,173,174,5,18,0,0,174,175,
        5,22,0,0,175,176,5,19,0,0,176,183,5,23,0,0,177,178,5,20,0,0,178,
        181,5,24,0,0,179,180,5,21,0,0,180,182,5,25,0,0,181,179,1,0,0,0,181,
        182,1,0,0,0,182,184,1,0,0,0,183,177,1,0,0,0,183,184,1,0,0,0,184,
        185,1,0,0,0,185,187,7,5,0,0,186,188,5,28,0,0,187,186,1,0,0,0,187,
        188,1,0,0,0,188,190,1,0,0,0,189,191,5,29,0,0,190,189,1,0,0,0,190,
        191,1,0,0,0,191,192,1,0,0,0,192,193,5,31,0,0,193,13,1,0,0,0,194,
        195,5,3,0,0,195,196,5,4,0,0,196,197,5,4,0,0,197,198,5,4,0,0,198,
        199,5,4,0,0,199,201,3,20,10,0,200,202,5,10,0,0,201,200,1,0,0,0,201,
        202,1,0,0,0,202,203,1,0,0,0,203,204,5,14,0,0,204,15,1,0,0,0,205,
        206,5,3,0,0,206,207,5,4,0,0,207,208,5,4,0,0,208,209,5,4,0,0,209,
        210,5,4,0,0,210,211,5,4,0,0,211,212,5,4,0,0,212,214,3,20,10,0,213,
        215,5,11,0,0,214,213,1,0,0,0,214,215,1,0,0,0,215,216,1,0,0,0,216,
        217,5,14,0,0,217,17,1,0,0,0,218,219,5,3,0,0,219,220,5,4,0,0,220,
        221,5,4,0,0,221,222,5,4,0,0,222,223,5,4,0,0,223,224,5,4,0,0,224,
        225,5,4,0,0,225,226,5,4,0,0,226,227,5,4,0,0,227,229,3,20,10,0,228,
        230,5,12,0,0,229,228,1,0,0,0,229,230,1,0,0,0,230,231,1,0,0,0,231,
        232,5,14,0,0,232,19,1,0,0,0,233,234,7,6,0,0,234,21,1,0,0,0,41,23,
        31,36,41,43,45,54,64,67,71,74,77,80,83,86,92,97,102,104,111,114,
        117,120,130,133,136,139,142,153,156,159,162,165,168,181,183,187,
        190,201,214,229
    ]

class VnmrPKParser ( Parser ):

    grammarFileName = "VnmrPKParser.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "'peak id.'", "'# Format:'", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "'Dim 0 (ppm)'", "'Dim 1 (ppm)'", 
                     "'Dim 2 (ppm)'", "'Dim 3 (ppm)'", "'Dev. 0'", "'Dev. 1'", 
                     "'Dev. 2'", "'Dev. 3'", "'Amplitude'", "<INVALID>", 
                     "<INVALID>", "'Assignment'", "<INVALID>", "<INVALID>", 
                     "'Peak_Number'", "'X(ppm)'", "'Y(ppm)'", "'Z(ppm)'", 
                     "'A(ppm)'", "<INVALID>", "<INVALID>", "'Linewidth_X(Hz)'", 
                     "'Linewidth_Y(Hz)'", "'Linewidth_Z(Hz)'", "'Linewidth_A(Hz)'", 
                     "'FWHM_X(Hz)'", "'FWHM_Y(Hz)'", "'FWHM_Z(Hz)'", "'FWHM_A(Hz)'", 
                     "'Comment'" ]

    symbolicNames = [ "<INVALID>", "Peak_id", "Format", "Integer", "Float", 
                      "Real", "COMMENT", "Double_quote_string", "EXCLM_COMMENT", 
                      "SMCLN_COMMENT", "Assignment_2d_ex", "Assignment_3d_ex", 
                      "Assignment_4d_ex", "SPACE", "RETURN", "ENCLOSE_COMMENT", 
                      "SECTION_COMMENT", "LINE_COMMENT", "Dim_0_ppm", "Dim_1_ppm", 
                      "Dim_2_ppm", "Dim_3_ppm", "Dev_0", "Dev_1", "Dev_2", 
                      "Dev_3", "Amplitude", "Intensity_LA", "Volume_LA", 
                      "Assignment", "SPACE_LA", "RETURN_LA", "Peak_number", 
                      "X_ppm", "Y_ppm", "Z_ppm", "A_ppm", "Intensity", "Volume", 
                      "Linewidth_X", "Linewidth_Y", "Linewidth_Z", "Linewidth_A", 
                      "FWHM_X", "FWHM_Y", "FWHM_Z", "FWHM_A", "Comment", 
                      "SPACE_FO", "RETURN_FO", "Any_name", "SPACE_CM", "RETURN_CM" ]

    RULE_vnmr_pk = 0
    RULE_comment = 1
    RULE_format = 2
    RULE_peak_ll2d = 3
    RULE_peak_ll3d = 4
    RULE_peak_ll4d = 5
    RULE_data_label = 6
    RULE_peak_2d = 7
    RULE_peak_3d = 8
    RULE_peak_4d = 9
    RULE_number = 10

    ruleNames =  [ "vnmr_pk", "comment", "format", "peak_ll2d", "peak_ll3d", 
                   "peak_ll4d", "data_label", "peak_2d", "peak_3d", "peak_4d", 
                   "number" ]

    EOF = Token.EOF
    Peak_id=1
    Format=2
    Integer=3
    Float=4
    Real=5
    COMMENT=6
    Double_quote_string=7
    EXCLM_COMMENT=8
    SMCLN_COMMENT=9
    Assignment_2d_ex=10
    Assignment_3d_ex=11
    Assignment_4d_ex=12
    SPACE=13
    RETURN=14
    ENCLOSE_COMMENT=15
    SECTION_COMMENT=16
    LINE_COMMENT=17
    Dim_0_ppm=18
    Dim_1_ppm=19
    Dim_2_ppm=20
    Dim_3_ppm=21
    Dev_0=22
    Dev_1=23
    Dev_2=24
    Dev_3=25
    Amplitude=26
    Intensity_LA=27
    Volume_LA=28
    Assignment=29
    SPACE_LA=30
    RETURN_LA=31
    Peak_number=32
    X_ppm=33
    Y_ppm=34
    Z_ppm=35
    A_ppm=36
    Intensity=37
    Volume=38
    Linewidth_X=39
    Linewidth_Y=40
    Linewidth_Z=41
    Linewidth_A=42
    FWHM_X=43
    FWHM_Y=44
    FWHM_Z=45
    FWHM_A=46
    Comment=47
    SPACE_FO=48
    RETURN_FO=49
    Any_name=50
    SPACE_CM=51
    RETURN_CM=52

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.13.0")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class Vnmr_pkContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EOF(self):
            return self.getToken(VnmrPKParser.EOF, 0)

        def RETURN(self):
            return self.getToken(VnmrPKParser.RETURN, 0)

        def comment(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(VnmrPKParser.CommentContext)
            else:
                return self.getTypedRuleContext(VnmrPKParser.CommentContext,i)


        def format_(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(VnmrPKParser.FormatContext)
            else:
                return self.getTypedRuleContext(VnmrPKParser.FormatContext,i)


        def data_label(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(VnmrPKParser.Data_labelContext)
            else:
                return self.getTypedRuleContext(VnmrPKParser.Data_labelContext,i)


        def peak_2d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(VnmrPKParser.Peak_2dContext)
            else:
                return self.getTypedRuleContext(VnmrPKParser.Peak_2dContext,i)


        def peak_3d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(VnmrPKParser.Peak_3dContext)
            else:
                return self.getTypedRuleContext(VnmrPKParser.Peak_3dContext,i)


        def peak_4d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(VnmrPKParser.Peak_4dContext)
            else:
                return self.getTypedRuleContext(VnmrPKParser.Peak_4dContext,i)


        def getRuleIndex(self):
            return VnmrPKParser.RULE_vnmr_pk

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterVnmr_pk" ):
                listener.enterVnmr_pk(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitVnmr_pk" ):
                listener.exitVnmr_pk(self)




    def vnmr_pk(self):

        localctx = VnmrPKParser.Vnmr_pkContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_vnmr_pk)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 23
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==14:
                self.state = 22
                self.match(VnmrPKParser.RETURN)


            self.state = 45
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & 78) != 0):
                self.state = 43
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,4,self._ctx)
                if la_ == 1:
                    self.state = 25
                    self.comment()
                    pass

                elif la_ == 2:
                    self.state = 26
                    self.format_()
                    pass

                elif la_ == 3:
                    self.state = 27
                    self.data_label()
                    pass

                elif la_ == 4:
                    self.state = 29 
                    self._errHandler.sync(self)
                    _alt = 1
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt == 1:
                            self.state = 28
                            self.peak_2d()

                        else:
                            raise NoViableAltException(self)
                        self.state = 31 
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,1,self._ctx)

                    pass

                elif la_ == 5:
                    self.state = 34 
                    self._errHandler.sync(self)
                    _alt = 1
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt == 1:
                            self.state = 33
                            self.peak_3d()

                        else:
                            raise NoViableAltException(self)
                        self.state = 36 
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,2,self._ctx)

                    pass

                elif la_ == 6:
                    self.state = 39 
                    self._errHandler.sync(self)
                    _alt = 1
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt == 1:
                            self.state = 38
                            self.peak_4d()

                        else:
                            raise NoViableAltException(self)
                        self.state = 41 
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,3,self._ctx)

                    pass


                self.state = 47
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 48
            self.match(VnmrPKParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class CommentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def COMMENT(self):
            return self.getToken(VnmrPKParser.COMMENT, 0)

        def RETURN_CM(self):
            return self.getToken(VnmrPKParser.RETURN_CM, 0)

        def EOF(self):
            return self.getToken(VnmrPKParser.EOF, 0)

        def Any_name(self, i:int=None):
            if i is None:
                return self.getTokens(VnmrPKParser.Any_name)
            else:
                return self.getToken(VnmrPKParser.Any_name, i)

        def getRuleIndex(self):
            return VnmrPKParser.RULE_comment

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterComment" ):
                listener.enterComment(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitComment" ):
                listener.exitComment(self)




    def comment(self):

        localctx = VnmrPKParser.CommentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_comment)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 50
            self.match(VnmrPKParser.COMMENT)
            self.state = 54
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==50:
                self.state = 51
                self.match(VnmrPKParser.Any_name)
                self.state = 56
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 57
            _la = self._input.LA(1)
            if not(_la==-1 or _la==52):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FormatContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Format(self):
            return self.getToken(VnmrPKParser.Format, 0)

        def Peak_number(self):
            return self.getToken(VnmrPKParser.Peak_number, 0)

        def X_ppm(self):
            return self.getToken(VnmrPKParser.X_ppm, 0)

        def Y_ppm(self):
            return self.getToken(VnmrPKParser.Y_ppm, 0)

        def Intensity(self):
            return self.getToken(VnmrPKParser.Intensity, 0)

        def RETURN_FO(self):
            return self.getToken(VnmrPKParser.RETURN_FO, 0)

        def Z_ppm(self):
            return self.getToken(VnmrPKParser.Z_ppm, 0)

        def A_ppm(self):
            return self.getToken(VnmrPKParser.A_ppm, 0)

        def Volume(self):
            return self.getToken(VnmrPKParser.Volume, 0)

        def Comment(self):
            return self.getToken(VnmrPKParser.Comment, 0)

        def Linewidth_X(self):
            return self.getToken(VnmrPKParser.Linewidth_X, 0)

        def FWHM_X(self):
            return self.getToken(VnmrPKParser.FWHM_X, 0)

        def Linewidth_Y(self):
            return self.getToken(VnmrPKParser.Linewidth_Y, 0)

        def FWHM_Y(self):
            return self.getToken(VnmrPKParser.FWHM_Y, 0)

        def Linewidth_Z(self):
            return self.getToken(VnmrPKParser.Linewidth_Z, 0)

        def FWHM_Z(self):
            return self.getToken(VnmrPKParser.FWHM_Z, 0)

        def Linewidth_A(self):
            return self.getToken(VnmrPKParser.Linewidth_A, 0)

        def FWHM_A(self):
            return self.getToken(VnmrPKParser.FWHM_A, 0)

        def peak_ll2d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(VnmrPKParser.Peak_ll2dContext)
            else:
                return self.getTypedRuleContext(VnmrPKParser.Peak_ll2dContext,i)


        def peak_ll3d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(VnmrPKParser.Peak_ll3dContext)
            else:
                return self.getTypedRuleContext(VnmrPKParser.Peak_ll3dContext,i)


        def peak_ll4d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(VnmrPKParser.Peak_ll4dContext)
            else:
                return self.getTypedRuleContext(VnmrPKParser.Peak_ll4dContext,i)


        def getRuleIndex(self):
            return VnmrPKParser.RULE_format

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFormat" ):
                listener.enterFormat(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFormat" ):
                listener.exitFormat(self)




    def format_(self):

        localctx = VnmrPKParser.FormatContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_format)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 59
            self.match(VnmrPKParser.Format)
            self.state = 60
            self.match(VnmrPKParser.Peak_number)
            self.state = 61
            self.match(VnmrPKParser.X_ppm)
            self.state = 62
            self.match(VnmrPKParser.Y_ppm)
            self.state = 64
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==35:
                self.state = 63
                self.match(VnmrPKParser.Z_ppm)


            self.state = 67
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==36:
                self.state = 66
                self.match(VnmrPKParser.A_ppm)


            self.state = 69
            self.match(VnmrPKParser.Intensity)
            self.state = 71
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==38:
                self.state = 70
                self.match(VnmrPKParser.Volume)


            self.state = 74
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==39 or _la==43:
                self.state = 73
                _la = self._input.LA(1)
                if not(_la==39 or _la==43):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()


            self.state = 77
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==40 or _la==44:
                self.state = 76
                _la = self._input.LA(1)
                if not(_la==40 or _la==44):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()


            self.state = 80
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==41 or _la==45:
                self.state = 79
                _la = self._input.LA(1)
                if not(_la==41 or _la==45):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()


            self.state = 83
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==42 or _la==46:
                self.state = 82
                _la = self._input.LA(1)
                if not(_la==42 or _la==46):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()


            self.state = 86
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==47:
                self.state = 85
                self.match(VnmrPKParser.Comment)


            self.state = 88
            self.match(VnmrPKParser.RETURN_FO)
            self.state = 104
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,18,self._ctx)
            if la_ == 1:
                self.state = 90 
                self._errHandler.sync(self)
                _alt = 1
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt == 1:
                        self.state = 89
                        self.peak_ll2d()

                    else:
                        raise NoViableAltException(self)
                    self.state = 92 
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,15,self._ctx)

                pass

            elif la_ == 2:
                self.state = 95 
                self._errHandler.sync(self)
                _alt = 1
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt == 1:
                        self.state = 94
                        self.peak_ll3d()

                    else:
                        raise NoViableAltException(self)
                    self.state = 97 
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,16,self._ctx)

                pass

            elif la_ == 3:
                self.state = 100 
                self._errHandler.sync(self)
                _alt = 1
                while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt == 1:
                        self.state = 99
                        self.peak_ll4d()

                    else:
                        raise NoViableAltException(self)
                    self.state = 102 
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,17,self._ctx)

                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_ll2dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self):
            return self.getToken(VnmrPKParser.Integer, 0)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(VnmrPKParser.Float)
            else:
                return self.getToken(VnmrPKParser.Float, i)

        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(VnmrPKParser.NumberContext)
            else:
                return self.getTypedRuleContext(VnmrPKParser.NumberContext,i)


        def RETURN(self):
            return self.getToken(VnmrPKParser.RETURN, 0)

        def Double_quote_string(self):
            return self.getToken(VnmrPKParser.Double_quote_string, 0)

        def getRuleIndex(self):
            return VnmrPKParser.RULE_peak_ll2d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_ll2d" ):
                listener.enterPeak_ll2d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_ll2d" ):
                listener.exitPeak_ll2d(self)




    def peak_ll2d(self):

        localctx = VnmrPKParser.Peak_ll2dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_peak_ll2d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 106
            self.match(VnmrPKParser.Integer)
            self.state = 107
            self.match(VnmrPKParser.Float)
            self.state = 108
            self.match(VnmrPKParser.Float)
            self.state = 109
            self.number()
            self.state = 111
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,19,self._ctx)
            if la_ == 1:
                self.state = 110
                self.number()


            self.state = 114
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,20,self._ctx)
            if la_ == 1:
                self.state = 113
                self.number()


            self.state = 117
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & 56) != 0):
                self.state = 116
                self.number()


            self.state = 120
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==7:
                self.state = 119
                self.match(VnmrPKParser.Double_quote_string)


            self.state = 122
            self.match(VnmrPKParser.RETURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_ll3dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self):
            return self.getToken(VnmrPKParser.Integer, 0)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(VnmrPKParser.Float)
            else:
                return self.getToken(VnmrPKParser.Float, i)

        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(VnmrPKParser.NumberContext)
            else:
                return self.getTypedRuleContext(VnmrPKParser.NumberContext,i)


        def RETURN(self):
            return self.getToken(VnmrPKParser.RETURN, 0)

        def Double_quote_string(self):
            return self.getToken(VnmrPKParser.Double_quote_string, 0)

        def getRuleIndex(self):
            return VnmrPKParser.RULE_peak_ll3d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_ll3d" ):
                listener.enterPeak_ll3d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_ll3d" ):
                listener.exitPeak_ll3d(self)




    def peak_ll3d(self):

        localctx = VnmrPKParser.Peak_ll3dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_peak_ll3d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 124
            self.match(VnmrPKParser.Integer)
            self.state = 125
            self.match(VnmrPKParser.Float)
            self.state = 126
            self.match(VnmrPKParser.Float)
            self.state = 127
            self.match(VnmrPKParser.Float)
            self.state = 128
            self.number()
            self.state = 130
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,23,self._ctx)
            if la_ == 1:
                self.state = 129
                self.number()


            self.state = 133
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,24,self._ctx)
            if la_ == 1:
                self.state = 132
                self.number()


            self.state = 136
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,25,self._ctx)
            if la_ == 1:
                self.state = 135
                self.number()


            self.state = 139
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & 56) != 0):
                self.state = 138
                self.number()


            self.state = 142
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==7:
                self.state = 141
                self.match(VnmrPKParser.Double_quote_string)


            self.state = 144
            self.match(VnmrPKParser.RETURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_ll4dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self):
            return self.getToken(VnmrPKParser.Integer, 0)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(VnmrPKParser.Float)
            else:
                return self.getToken(VnmrPKParser.Float, i)

        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(VnmrPKParser.NumberContext)
            else:
                return self.getTypedRuleContext(VnmrPKParser.NumberContext,i)


        def RETURN(self):
            return self.getToken(VnmrPKParser.RETURN, 0)

        def Double_quote_string(self):
            return self.getToken(VnmrPKParser.Double_quote_string, 0)

        def getRuleIndex(self):
            return VnmrPKParser.RULE_peak_ll4d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_ll4d" ):
                listener.enterPeak_ll4d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_ll4d" ):
                listener.exitPeak_ll4d(self)




    def peak_ll4d(self):

        localctx = VnmrPKParser.Peak_ll4dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_peak_ll4d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 146
            self.match(VnmrPKParser.Integer)
            self.state = 147
            self.match(VnmrPKParser.Float)
            self.state = 148
            self.match(VnmrPKParser.Float)
            self.state = 149
            self.match(VnmrPKParser.Float)
            self.state = 150
            self.match(VnmrPKParser.Float)
            self.state = 151
            self.number()
            self.state = 153
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,28,self._ctx)
            if la_ == 1:
                self.state = 152
                self.number()


            self.state = 156
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,29,self._ctx)
            if la_ == 1:
                self.state = 155
                self.number()


            self.state = 159
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,30,self._ctx)
            if la_ == 1:
                self.state = 158
                self.number()


            self.state = 162
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,31,self._ctx)
            if la_ == 1:
                self.state = 161
                self.number()


            self.state = 165
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if (((_la) & ~0x3f) == 0 and ((1 << _la) & 56) != 0):
                self.state = 164
                self.number()


            self.state = 168
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==7:
                self.state = 167
                self.match(VnmrPKParser.Double_quote_string)


            self.state = 170
            self.match(VnmrPKParser.RETURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Data_labelContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Peak_id(self):
            return self.getToken(VnmrPKParser.Peak_id, 0)

        def Dim_0_ppm(self):
            return self.getToken(VnmrPKParser.Dim_0_ppm, 0)

        def Dev_0(self):
            return self.getToken(VnmrPKParser.Dev_0, 0)

        def Dim_1_ppm(self):
            return self.getToken(VnmrPKParser.Dim_1_ppm, 0)

        def Dev_1(self):
            return self.getToken(VnmrPKParser.Dev_1, 0)

        def RETURN_LA(self):
            return self.getToken(VnmrPKParser.RETURN_LA, 0)

        def Amplitude(self):
            return self.getToken(VnmrPKParser.Amplitude, 0)

        def Intensity_LA(self):
            return self.getToken(VnmrPKParser.Intensity_LA, 0)

        def Dim_2_ppm(self):
            return self.getToken(VnmrPKParser.Dim_2_ppm, 0)

        def Dev_2(self):
            return self.getToken(VnmrPKParser.Dev_2, 0)

        def Volume_LA(self):
            return self.getToken(VnmrPKParser.Volume_LA, 0)

        def Assignment(self):
            return self.getToken(VnmrPKParser.Assignment, 0)

        def Dim_3_ppm(self):
            return self.getToken(VnmrPKParser.Dim_3_ppm, 0)

        def Dev_3(self):
            return self.getToken(VnmrPKParser.Dev_3, 0)

        def getRuleIndex(self):
            return VnmrPKParser.RULE_data_label

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterData_label" ):
                listener.enterData_label(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitData_label" ):
                listener.exitData_label(self)




    def data_label(self):

        localctx = VnmrPKParser.Data_labelContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_data_label)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 172
            self.match(VnmrPKParser.Peak_id)
            self.state = 173
            self.match(VnmrPKParser.Dim_0_ppm)
            self.state = 174
            self.match(VnmrPKParser.Dev_0)
            self.state = 175
            self.match(VnmrPKParser.Dim_1_ppm)
            self.state = 176
            self.match(VnmrPKParser.Dev_1)
            self.state = 183
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==20:
                self.state = 177
                self.match(VnmrPKParser.Dim_2_ppm)
                self.state = 178
                self.match(VnmrPKParser.Dev_2)
                self.state = 181
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==21:
                    self.state = 179
                    self.match(VnmrPKParser.Dim_3_ppm)
                    self.state = 180
                    self.match(VnmrPKParser.Dev_3)




            self.state = 185
            _la = self._input.LA(1)
            if not(_la==26 or _la==27):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
            self.state = 187
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==28:
                self.state = 186
                self.match(VnmrPKParser.Volume_LA)


            self.state = 190
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==29:
                self.state = 189
                self.match(VnmrPKParser.Assignment)


            self.state = 192
            self.match(VnmrPKParser.RETURN_LA)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_2dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self):
            return self.getToken(VnmrPKParser.Integer, 0)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(VnmrPKParser.Float)
            else:
                return self.getToken(VnmrPKParser.Float, i)

        def number(self):
            return self.getTypedRuleContext(VnmrPKParser.NumberContext,0)


        def RETURN(self):
            return self.getToken(VnmrPKParser.RETURN, 0)

        def Assignment_2d_ex(self):
            return self.getToken(VnmrPKParser.Assignment_2d_ex, 0)

        def getRuleIndex(self):
            return VnmrPKParser.RULE_peak_2d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_2d" ):
                listener.enterPeak_2d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_2d" ):
                listener.exitPeak_2d(self)




    def peak_2d(self):

        localctx = VnmrPKParser.Peak_2dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_peak_2d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 194
            self.match(VnmrPKParser.Integer)
            self.state = 195
            self.match(VnmrPKParser.Float)
            self.state = 196
            self.match(VnmrPKParser.Float)
            self.state = 197
            self.match(VnmrPKParser.Float)
            self.state = 198
            self.match(VnmrPKParser.Float)
            self.state = 199
            self.number()
            self.state = 201
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==10:
                self.state = 200
                self.match(VnmrPKParser.Assignment_2d_ex)


            self.state = 203
            self.match(VnmrPKParser.RETURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_3dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self):
            return self.getToken(VnmrPKParser.Integer, 0)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(VnmrPKParser.Float)
            else:
                return self.getToken(VnmrPKParser.Float, i)

        def number(self):
            return self.getTypedRuleContext(VnmrPKParser.NumberContext,0)


        def RETURN(self):
            return self.getToken(VnmrPKParser.RETURN, 0)

        def Assignment_3d_ex(self):
            return self.getToken(VnmrPKParser.Assignment_3d_ex, 0)

        def getRuleIndex(self):
            return VnmrPKParser.RULE_peak_3d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_3d" ):
                listener.enterPeak_3d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_3d" ):
                listener.exitPeak_3d(self)




    def peak_3d(self):

        localctx = VnmrPKParser.Peak_3dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_peak_3d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 205
            self.match(VnmrPKParser.Integer)
            self.state = 206
            self.match(VnmrPKParser.Float)
            self.state = 207
            self.match(VnmrPKParser.Float)
            self.state = 208
            self.match(VnmrPKParser.Float)
            self.state = 209
            self.match(VnmrPKParser.Float)
            self.state = 210
            self.match(VnmrPKParser.Float)
            self.state = 211
            self.match(VnmrPKParser.Float)
            self.state = 212
            self.number()
            self.state = 214
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==11:
                self.state = 213
                self.match(VnmrPKParser.Assignment_3d_ex)


            self.state = 216
            self.match(VnmrPKParser.RETURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_4dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self):
            return self.getToken(VnmrPKParser.Integer, 0)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(VnmrPKParser.Float)
            else:
                return self.getToken(VnmrPKParser.Float, i)

        def number(self):
            return self.getTypedRuleContext(VnmrPKParser.NumberContext,0)


        def RETURN(self):
            return self.getToken(VnmrPKParser.RETURN, 0)

        def Assignment_4d_ex(self):
            return self.getToken(VnmrPKParser.Assignment_4d_ex, 0)

        def getRuleIndex(self):
            return VnmrPKParser.RULE_peak_4d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_4d" ):
                listener.enterPeak_4d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_4d" ):
                listener.exitPeak_4d(self)




    def peak_4d(self):

        localctx = VnmrPKParser.Peak_4dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_peak_4d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 218
            self.match(VnmrPKParser.Integer)
            self.state = 219
            self.match(VnmrPKParser.Float)
            self.state = 220
            self.match(VnmrPKParser.Float)
            self.state = 221
            self.match(VnmrPKParser.Float)
            self.state = 222
            self.match(VnmrPKParser.Float)
            self.state = 223
            self.match(VnmrPKParser.Float)
            self.state = 224
            self.match(VnmrPKParser.Float)
            self.state = 225
            self.match(VnmrPKParser.Float)
            self.state = 226
            self.match(VnmrPKParser.Float)
            self.state = 227
            self.number()
            self.state = 229
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==12:
                self.state = 228
                self.match(VnmrPKParser.Assignment_4d_ex)


            self.state = 231
            self.match(VnmrPKParser.RETURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class NumberContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Real(self):
            return self.getToken(VnmrPKParser.Real, 0)

        def Float(self):
            return self.getToken(VnmrPKParser.Float, 0)

        def Integer(self):
            return self.getToken(VnmrPKParser.Integer, 0)

        def getRuleIndex(self):
            return VnmrPKParser.RULE_number

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNumber" ):
                listener.enterNumber(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNumber" ):
                listener.exitNumber(self)




    def number(self):

        localctx = VnmrPKParser.NumberContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_number)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 233
            _la = self._input.LA(1)
            if not((((_la) & ~0x3f) == 0 and ((1 << _la) & 56) != 0)):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx








##
# File: VnmrPKParserListener.py
# Date: 16-Dec-2024
#
# Updates:
""" ParserLister class for VNMR PK files.
    @author: Masashi Yokochi
"""
__docformat__ = "restructuredtext en"
__author__ = "Masashi Yokochi"
__email__ = "yokochi@protein.osaka-u.ac.jp"
__license__ = "Apache License 2.0"
__version__ = "1.0.0"

import sys
import copy
import re

from antlr4 import ParseTreeListener
from typing import IO, List, Optional

try:
    from wwpdb.utils.nmr.io.CifReader import CifReader
    from wwpdb.utils.nmr.pk.VnmrPKParser import VnmrPKParser
    from wwpdb.utils.nmr.pk.BasePKParserListener import BasePKParserListener
    from wwpdb.utils.nmr.mr.ParserListenerUtil import (REPRESENTATIVE_MODEL_ID,
                                                       REPRESENTATIVE_ALT_ID,
                                                       SPECTRAL_DIM_TEMPLATE,
                                                       ISOTOPE_NUMBERS_OF_NMR_OBS_NUCS)
    from wwpdb.utils.nmr.ChemCompUtil import ChemCompUtil
    from wwpdb.utils.nmr.BMRBChemShiftStat import BMRBChemShiftStat
    from wwpdb.utils.nmr.nef.NEFTranslator import NEFTranslator
    from wwpdb.utils.nmr.AlignUtil import emptyValue
except ImportError:
    from nmr.io.CifReader import CifReader
    from nmr.pk.VnmrPKParser import VnmrPKParser
    from nmr.pk.BasePKParserListener import BasePKParserListener
    from nmr.mr.ParserListenerUtil import (REPRESENTATIVE_MODEL_ID,
                                           REPRESENTATIVE_ALT_ID,
                                           SPECTRAL_DIM_TEMPLATE,
                                           ISOTOPE_NUMBERS_OF_NMR_OBS_NUCS)
    from nmr.ChemCompUtil import ChemCompUtil
    from nmr.BMRBChemShiftStat import BMRBChemShiftStat
    from nmr.nef.NEFTranslator import NEFTranslator
    from nmr.AlignUtil import emptyValue


# This class defines a complete listener for a parse tree produced by VnmrPKParser.
class VnmrPKParserListener(ParseTreeListener, BasePKParserListener):

    __spectrum_names = None
    __has_volume = False
    __has_line_width = False
    __has_assign = False

    def __init__(self, verbose: bool = True, log: IO = sys.stdout,
                 representativeModelId: int = REPRESENTATIVE_MODEL_ID,
                 representativeAltId: str = REPRESENTATIVE_ALT_ID,
                 mrAtomNameMapping: Optional[List[dict]] = None,
                 cR: Optional[CifReader] = None, caC: Optional[dict] = None, ccU: Optional[ChemCompUtil] = None,
                 csStat: Optional[BMRBChemShiftStat] = None, nefT: Optional[NEFTranslator] = None,
                 reasons: Optional[dict] = None):
        super().__init__(verbose, log, representativeModelId, representativeAltId,
                         mrAtomNameMapping, cR, caC, ccU, csStat, nefT, reasons)

        self.file_type = 'nm-pea-vnm'
        self.software_name = 'VNMR'

    # Enter a parse tree produced by VnmrPKParser#vnmr_pk.
    def enterVnmr_pk(self, ctx: VnmrPKParser.Vnmr_pkContext):  # pylint: disable=unused-argument
        self.__spectrum_names = {}

        self.enter()

    # Exit a parse tree produced by VnmrPKParser#vnmr_pk.
    def exitVnmr_pk(self, ctx: VnmrPKParser.Vnmr_pkContext):  # pylint: disable=unused-argument
        self.exit(self.__spectrum_names if len(self.__spectrum_names) > 0 else None)

    # Enter a parse tree produced by VnmrPKParser#comment.
    def enterComment(self, ctx: VnmrPKParser.CommentContext):  # pylint: disable=unused-argument
        pass

    # Exit a parse tree produced by VnmrPKParser#comment.
    def exitComment(self, ctx: VnmrPKParser.CommentContext):
        comment = []
        for col in range(20):
            if ctx.Any_name(col):
                text = str(ctx.Any_name(col))
                if text[0] in ('#', '!'):
                    break
                if text[0] in ('>', '<'):
                    continue
                comment.append(str(ctx.Any_name(col)))
            else:
                break
        last_comment = None if len(comment) == 0 else ' '.join(comment)

        if last_comment is None:
            return

        if last_comment.startswith('Spectrum: '):
            self.spectrum_name = last_comment.split()[1]

        elif last_comment.startswith('X-Axis: '):
            axis_names = last_comment.split(',')

            self.num_of_dim = len(axis_names)
            labels = [n.split()[1] for n in axis_names]

            for _dim_id, _axis_code in enumerate(labels, start=1):

                if _dim_id not in self.cur_spectral_dim:
                    cur_spectral_dim = copy.copy(SPECTRAL_DIM_TEMPLATE)
                else:
                    cur_spectral_dim = self.cur_spectral_dim[_dim_id]

                cur_spectral_dim['axis_code'] = _axis_code

                digits = re.findall(r'\d+', _axis_code)
                for digit in digits:
                    num = int(digit)
                    nuc = next((k for k, v in ISOTOPE_NUMBERS_OF_NMR_OBS_NUCS.items() if num == v[0]), None)
                    if nuc is not None:
                        cur_spectral_dim['atom_type'] = nuc
                        cur_spectral_dim['atom_isotope_number'] = num
                        break
                if cur_spectral_dim['atom_type'] is None:
                    for a in _axis_code:
                        a = a.upper()
                        if a in ISOTOPE_NUMBERS_OF_NMR_OBS_NUCS:
                            cur_spectral_dim['atom_type'] = a
                            cur_spectral_dim['atom_isotope_number'] = ISOTOPE_NUMBERS_OF_NMR_OBS_NUCS[a][0]
                            break

                isotope_number = cur_spectral_dim['atom_isotope_number']

                cur_spectral_dim['acquisition'] = 'yes' if _dim_id == self.acq_dim_id\
                    and (isotope_number == 1 or (isotope_number == 13 and self.exptlMethod == 'SOLID-STATE NMR')) else 'no'

                if _dim_id == 1 and cur_spectral_dim['acquisition'] == 'no':
                    self.acq_dim_id = self.num_of_dim

                cur_spectral_dim['under_sampling_type'] = 'not observed' if cur_spectral_dim['acquisition'] == 'yes' else 'aliased'

                self.cur_spectral_dim[_dim_id] = cur_spectral_dim

    # Enter a parse tree produced by VnmrPKParser#format.
    def enterFormat(self, ctx: VnmrPKParser.FormatContext):
        if ctx.A_ppm():
            self.num_of_dim = 4
            self.initSpectralDim()
        elif ctx.Z_ppm():
            self.num_of_dim = 3
            self.initSpectralDim()
        elif ctx.Y_ppm():
            self.num_of_dim = 2
            self.initSpectralDim()

        if self.spectrum_name is not None:
            if self.num_of_dim not in self.__spectrum_names:
                self.__spectrum_names[self.num_of_dim] = {}
            if self.cur_list_id not in self.__spectrum_names[self.num_of_dim]:
                self.__spectrum_names[self.num_of_dim][self.cur_list_id] = self.spectrum_name

        self.__has_volume = bool(ctx.Volume())
        self.__has_line_width = bool(ctx.Linewidth_X()) or bool(ctx.FWHM_X())
        self.__has_assign = bool(ctx.Comment())

    # Exit a parse tree produced by VnmrPKParser#format.
    def exitFormat(self, ctx: VnmrPKParser.FormatContext):  # pylint: disable=unused-argument
        pass

    # Enter a parse tree produced by VnmrPKParser#peak_ll2d.
    def enterPeak_ll2d(self, ctx: VnmrPKParser.Peak_ll2dContext):  # pylint: disable=unused-argument
        self.peaks2D += 1

        self.atomSelectionSets.clear()
        self.asIsSets.clear()

    # Exit a parse tree produced by VnmrPKParser#peak_ll2d.
    def exitPeak_ll2d(self, ctx: VnmrPKParser.Peak_ll2dContext):

        try:

            if len(self.numberSelection) == 0:
                self.peaks2D -= 1
                return

            index = int(str(ctx.Integer()))

            ass = None
            if self.__has_assign and ctx.Double_quote_string():
                ass = str(ctx.Double_quote_string())[1:-1].strip()
                if ass in emptyValue:
                    ass = None

            x_ppm = float(str(ctx.Float(0)))
            y_ppm = float(str(ctx.Float(1)))

            height = self.originalNumberSelection[0]

            offset = 1

            volume = None
            if self.__has_volume:
                volume = self.originalNumberSelection[offset]
                offset += 1

            if self.__has_line_width:
                x_lw_hz = self.numberSelection[offset]
                y_lw_hz = self.numberSelection[offset + 1]

            if not self.hasPolySeq and not self.hasNonPolySeq:
                return

            if None in (x_ppm, y_ppm):
                self.peaks2D -= 1
                return

            dstFunc = self.validatePeak2D(index, x_ppm, y_ppm, None, None, None, None,
                                          None, None, x_lw_hz, y_lw_hz, height, None, volume, None)

            if dstFunc is None:
                self.peaks2D -= 1
                return

            cur_spectral_dim = self.spectral_dim[self.num_of_dim][self.cur_list_id]

            cur_spectral_dim[1]['freq_hint'].append(x_ppm)
            cur_spectral_dim[2]['freq_hint'].append(y_ppm)

            has_assignments = has_multiple_assignments = False
            asis1 = asis2 = None

            if ass is not None:
                assignments = []
                if len(ass.split('-')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split('-'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(':')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(':'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(';')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(';'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(',')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(','):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                else:
                    assignments = [None] * self.num_of_dim
                    _assignments = self.extractPeakAssignment(self.num_of_dim, ass, index)
                    if _assignments is not None and len(_assignments) == self.num_of_dim:
                        for idx in range(self.num_of_dim):
                            assignments[idx] = [_assignments[idx]]  # pylint: disable=unsubscriptable-object

                has_assignments, has_multiple_assignments, asis1, asis2 =\
                    self.checkAssignments2D(index, assignments)

            self.addAssignedPkRow2D(index, dstFunc, has_assignments, has_multiple_assignments,
                                    asis1, asis2,
                                    f'{ass} -> ', None if has_assignments and not has_multiple_assignments else ass)

        finally:
            self.numberSelection.clear()
            self.originalNumberSelection.clear()

    # Enter a parse tree produced by VnmrPKParser#peak_ll3d.
    def enterPeak_ll3d(self, ctx: VnmrPKParser.Peak_ll3dContext):  # pylint: disable=unused-argument
        self.peaks3D += 1

        self.atomSelectionSets.clear()
        self.asIsSets.clear()

    # Exit a parse tree produced by VnmrPKParser#peak_ll3d.
    def exitPeak_ll3d(self, ctx: VnmrPKParser.Peak_ll3dContext):

        try:

            if len(self.numberSelection) == 0:
                self.peaks3D -= 1
                return

            index = int(str(ctx.Integer()))

            ass = None
            if self.__has_assign and ctx.Double_quote_string():
                ass = str(ctx.Double_quote_string())[1:-1].strip()
                if ass in emptyValue:
                    ass = None

            x_ppm = float(str(ctx.Float(0)))
            y_ppm = float(str(ctx.Float(1)))
            z_ppm = float(str(ctx.Float(2)))

            height = self.originalNumberSelection[0]

            offset = 1

            volume = None
            if self.__has_volume:
                volume = self.originalNumberSelection[offset]
                offset += 1

            if self.__has_line_width:
                x_lw_hz = self.numberSelection[offset]
                y_lw_hz = self.numberSelection[offset + 1]
                z_lw_hz = self.numberSelection[offset + 2]

            if None in (x_ppm, y_ppm, z_ppm):
                self.peaks3D -= 1
                return

            dstFunc = self.validatePeak3D(index, x_ppm, y_ppm, z_ppm, None, None, None, None, None, None,
                                          None, None, None, x_lw_hz, y_lw_hz, z_lw_hz, height, None, volume, None)

            if dstFunc is None:
                self.peaks3D -= 1
                return

            cur_spectral_dim = self.spectral_dim[self.num_of_dim][self.cur_list_id]

            cur_spectral_dim[1]['freq_hint'].append(x_ppm)
            cur_spectral_dim[2]['freq_hint'].append(y_ppm)
            cur_spectral_dim[3]['freq_hint'].append(z_ppm)

            has_assignments = has_multiple_assignments = False
            asis1 = asis2 = asis3 = None

            if ass is not None:
                assignments = []
                if len(ass.split('-')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split('-'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(':')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(':'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(';')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(';'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(',')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(','):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                else:
                    assignments = [None] * self.num_of_dim
                    _assignments = self.extractPeakAssignment(self.num_of_dim, ass, index)
                    if _assignments is not None and len(_assignments) == self.num_of_dim:
                        for idx in range(self.num_of_dim):
                            assignments[idx] = [_assignments[idx]]  # pylint: disable=unsubscriptable-object

                has_assignments, has_multiple_assignments, asis1, asis2, asis3 =\
                    self.checkAssignments3D(index, assignments)

            self.addAssignedPkRow3D(index, dstFunc, has_assignments, has_multiple_assignments,
                                    asis1, asis2, asis3,
                                    f'{ass} -> ', None if has_assignments and not has_multiple_assignments else ass)

        finally:
            self.numberSelection.clear()
            self.originalNumberSelection.clear()

    # Enter a parse tree produced by VnmrPKParser#peak_ll4d.
    def enterPeak_ll4d(self, ctx: VnmrPKParser.Peak_ll4dContext):  # pylint: disable=unused-argument
        self.peaks4D += 1

        self.atomSelectionSets.clear()
        self.asIsSets.clear()

    # Exit a parse tree produced by VnmrPKParser#peak_ll4d.
    def exitPeak_ll4d(self, ctx: VnmrPKParser.Peak_ll4dContext):

        try:

            if len(self.numberSelection) == 0:
                self.peaks4D -= 1
                return

            index = int(str(ctx.Integer()))

            ass = None
            if self.__has_assign and ctx.Double_quote_string():
                ass = str(ctx.Double_quote_string())[1:-1].strip()
                if ass in emptyValue:
                    ass = None

            x_ppm = float(str(ctx.Float(0)))
            y_ppm = float(str(ctx.Float(1)))
            z_ppm = float(str(ctx.Float(2)))
            a_ppm = float(str(ctx.Float(3)))

            height = self.originalNumberSelection[0]

            offset = 1

            volume = None
            if self.__has_volume:
                volume = self.originalNumberSelection[offset]
                offset += 1

            if self.__has_line_width:
                x_lw_hz = self.numberSelection[offset]
                y_lw_hz = self.numberSelection[offset + 1]
                z_lw_hz = self.numberSelection[offset + 2]
                a_lw_hz = self.numberSelection[offset + 3]

            if not self.hasPolySeq and not self.hasNonPolySeq:
                return

            if None in (x_ppm, y_ppm, z_ppm, a_ppm):
                self.peaks4D -= 1
                return

            dstFunc = self.validatePeak4D(index, x_ppm, y_ppm, z_ppm, a_ppm, None, None, None, None, None, None, None, None,
                                          None, None, None, None, x_lw_hz, y_lw_hz, z_lw_hz, a_lw_hz, height, None, volume, None)

            if dstFunc is None:
                self.peaks4D -= 1
                return

            cur_spectral_dim = self.spectral_dim[self.num_of_dim][self.cur_list_id]

            cur_spectral_dim[1]['freq_hint'].append(x_ppm)
            cur_spectral_dim[2]['freq_hint'].append(y_ppm)
            cur_spectral_dim[3]['freq_hint'].append(z_ppm)
            cur_spectral_dim[4]['freq_hint'].append(a_ppm)

            has_assignments = has_multiple_assignments = False
            asis1 = asis2 = asis3 = asis4 = None

            if ass is not None:
                assignments = []
                if len(ass.split('-')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split('-'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(':')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(':'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(';')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(';'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(',')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(','):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                else:
                    assignments = [None] * self.num_of_dim
                    _assignments = self.extractPeakAssignment(self.num_of_dim, ass, index)
                    if _assignments is not None and len(_assignments) == self.num_of_dim:
                        for idx in range(self.num_of_dim):
                            assignments[idx] = [_assignments[idx]]  # pylint: disable=unsubscriptable-object

                has_assignments, has_multiple_assignments, asis1, asis2, asis3, asis4 =\
                    self.checkAssignments4D(index, assignments)

            self.addAssignedPkRow4D(index, dstFunc, has_assignments, has_multiple_assignments,
                                    asis1, asis2, asis3, asis4,
                                    f'{ass} -> ', None if has_assignments and not has_multiple_assignments else ass)

        finally:
            self.numberSelection.clear()
            self.originalNumberSelection.clear()

    # Enter a parse tree produced by VnmrPKParser#data_label.
    def enterData_label(self, ctx: VnmrPKParser.Data_labelContext):
        if ctx.Dim_1_ppm():
            self.num_of_dim = max(self.num_of_dim, 2)
        if ctx.Dim_2_ppm():
            self.num_of_dim = max(self.num_of_dim, 3)
        if ctx.Dim_3_ppm():
            self.num_of_dim = max(self.num_of_dim, 4)

        self.__has_volume = bool(ctx.Volume_LA())
        self.__has_assign = bool(ctx.Assignment())

        self.initSpectralDim()

    # Exit a parse tree produced by VnmrPKParser#data_label.
    def exitData_label(self, ctx: VnmrPKParser.Data_labelContext):  # pylint: disable=unused-argument
        pass

    # Enter a parse tree produced by VnmrPKParser#peak_2d.
    def enterPeak_2d(self, ctx: VnmrPKParser.Peak_2dContext):  # pylint: disable=unused-argument
        if self.cur_subtype != 'peak2d':
            self.num_of_dim = 2
            self.initSpectralDim()

        self.peaks2D += 1

        self.atomSelectionSets.clear()
        self.asIsSets.clear()
        self.spectrum_name = None

    # Exit a parse tree produced by VnmrPKParser#peak_2d.
    def exitPeak_2d(self, ctx: VnmrPKParser.Peak_2dContext):

        try:

            if len(self.numberSelection) == 0:
                self.peaks2D -= 1
                return

            index = int(str(ctx.Integer()))

            ass = None
            if self.__has_assign and ctx.Assignment_2d_ex():
                ass = str(ctx.Assignment_2d_ex())
                if '?' in ass:
                    ass = None

            x_ppm = float(str(ctx.Float(0)))
            x_dev = abs(float(str(ctx.Float(1))))
            y_ppm = float(str(ctx.Float(2)))
            y_dev = abs(float(str(ctx.Float(3))))

            height = self.originalNumberSelection[0]
            volume = None
            if self.__has_volume:
                volume = self.originalNumberSelection[1]

            if not self.hasPolySeq and not self.hasNonPolySeq:
                return

            if None in (x_ppm, y_ppm):
                self.peaks2D -= 1
                return

            dstFunc = self.validatePeak2D(index, x_ppm, y_ppm, x_dev, y_dev, None, None,
                                          None, None, None, None, height, None, volume, None)

            if dstFunc is None:
                self.peaks2D -= 1
                return

            cur_spectral_dim = self.spectral_dim[self.num_of_dim][self.cur_list_id]

            cur_spectral_dim[1]['freq_hint'].append(x_ppm)
            cur_spectral_dim[2]['freq_hint'].append(y_ppm)

            has_assignments = has_multiple_assignments = False
            asis1 = asis2 = None

            if ass is not None:
                assignments = []
                if len(ass.split('-')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split('-'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(':')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(':'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(';')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(';'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(',')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(','):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                else:
                    assignments = [None] * self.num_of_dim
                    _assignments = self.extractPeakAssignment(self.num_of_dim, ass, index)
                    if _assignments is not None and len(_assignments) == self.num_of_dim:
                        for idx in range(self.num_of_dim):
                            assignments[idx] = [_assignments[idx]]  # pylint: disable=unsubscriptable-object

                has_assignments, has_multiple_assignments, asis1, asis2 =\
                    self.checkAssignments2D(index, assignments)

            self.addAssignedPkRow2D(index, dstFunc, has_assignments, has_multiple_assignments,
                                    asis1, asis2,
                                    f'{ass} -> ', None if has_assignments and not has_multiple_assignments else ass)

        finally:
            self.numberSelection.clear()
            self.originalNumberSelection.clear()

    # Enter a parse tree produced by VnmrPKParser#peak_3d.
    def enterPeak_3d(self, ctx: VnmrPKParser.Peak_3dContext):  # pylint: disable=unused-argument
        if self.cur_subtype != 'peak3d':
            self.num_of_dim = 3
            self.initSpectralDim()

        self.peaks3D += 1

        self.atomSelectionSets.clear()
        self.asIsSets.clear()
        self.spectrum_name = None

    # Exit a parse tree produced by VnmrPKParser#peak_3d.
    def exitPeak_3d(self, ctx: VnmrPKParser.Peak_3dContext):

        try:

            if len(self.numberSelection) == 0:
                self.peaks3D -= 1
                return

            index = int(str(ctx.Integer()))

            ass = None
            if self.__has_assign and ctx.Assignment_3d_ex():
                ass = str(ctx.Assignment_3d_ex())
                if '?' in ass:
                    ass = None

            x_ppm = float(str(ctx.Float(0)))
            x_dev = abs(float(str(ctx.Float(1))))
            y_ppm = float(str(ctx.Float(2)))
            y_dev = abs(float(str(ctx.Float(3))))
            z_ppm = float(str(ctx.Float(4)))
            z_dev = abs(float(str(ctx.Float(5))))

            height = self.originalNumberSelection[0]
            volume = None
            if self.__has_volume:
                volume = self.originalNumberSelection[1]

            if not self.hasPolySeq and not self.hasNonPolySeq:
                return

            if None in (x_ppm, y_ppm, z_ppm):
                self.peaks3D -= 1
                return

            dstFunc = self.validatePeak3D(index, x_ppm, y_ppm, z_ppm, x_dev, y_dev, z_dev, None, None, None,
                                          None, None, None, None, None, None, height, None, volume, None)

            if dstFunc is None:
                self.peaks3D -= 1
                return

            cur_spectral_dim = self.spectral_dim[self.num_of_dim][self.cur_list_id]

            cur_spectral_dim[1]['freq_hint'].append(x_ppm)
            cur_spectral_dim[2]['freq_hint'].append(y_ppm)
            cur_spectral_dim[3]['freq_hint'].append(z_ppm)

            has_assignments = has_multiple_assignments = False
            asis1 = asis2 = asis3 = None

            if ass is not None:
                assignments = []
                if len(ass.split('-')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split('-'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(':')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(':'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(';')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(';'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(',')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(','):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                else:
                    assignments = [None] * self.num_of_dim
                    _assignments = self.extractPeakAssignment(self.num_of_dim, ass, index)
                    if _assignments is not None and len(_assignments) == self.num_of_dim:
                        for idx in range(self.num_of_dim):
                            assignments[idx] = [_assignments[idx]]  # pylint: disable=unsubscriptable-object

                has_assignments, has_multiple_assignments, asis1, asis2, asis3 =\
                    self.checkAssignments3D(index, assignments)

            self.addAssignedPkRow3D(index, dstFunc, has_assignments, has_multiple_assignments,
                                    asis1, asis2, asis3,
                                    f'{ass} -> ', None if has_assignments and not has_multiple_assignments else ass)

        finally:
            self.numberSelection.clear()
            self.originalNumberSelection.clear()

    # Enter a parse tree produced by VnmrPKParser#peak_4d.
    def enterPeak_4d(self, ctx: VnmrPKParser.Peak_4dContext):  # pylint: disable=unused-argument
        if self.cur_subtype != 'peak4d':
            self.num_of_dim = 4
            self.initSpectralDim()

        self.peaks4D += 1

        self.atomSelectionSets.clear()
        self.asIsSets.clear()
        self.spectrum_name = None

    # Exit a parse tree produced by VnmrPKParser#peak_4d.
    def exitPeak_4d(self, ctx: VnmrPKParser.Peak_4dContext):

        try:

            if len(self.numberSelection) == 0:
                self.peaks4D -= 1
                return

            index = int(str(ctx.Integer()))

            ass = None
            if self.__has_assign and ctx.Assignment_4d_ex():
                ass = str(ctx.Assignment_4d_ex())
                if '?' in ass:
                    ass = None

            x_ppm = float(str(ctx.Float(0)))
            x_dev = abs(float(str(ctx.Float(1))))
            y_ppm = float(str(ctx.Float(2)))
            y_dev = abs(float(str(ctx.Float(3))))
            z_ppm = float(str(ctx.Float(4)))
            z_dev = abs(float(str(ctx.Float(5))))
            a_ppm = float(str(ctx.Float(6)))
            a_dev = abs(float(str(ctx.Float(7))))

            height = self.originalNumberSelection[0]
            volume = None
            if self.__has_volume:
                volume = self.originalNumberSelection[1]

            if not self.hasPolySeq and not self.hasNonPolySeq:
                return

            if None in (x_ppm, y_ppm, z_ppm, a_ppm):
                self.peaks4D -= 1
                return

            dstFunc = self.validatePeak4D(index, x_ppm, y_ppm, z_ppm, a_ppm, x_dev, y_dev, z_dev, a_dev, None, None, None, None,
                                          None, None, None, None, None, None, None, None, height, None, volume, None)

            if dstFunc is None:
                self.peaks4D -= 1
                return

            cur_spectral_dim = self.spectral_dim[self.num_of_dim][self.cur_list_id]

            cur_spectral_dim[1]['freq_hint'].append(x_ppm)
            cur_spectral_dim[2]['freq_hint'].append(y_ppm)
            cur_spectral_dim[3]['freq_hint'].append(z_ppm)
            cur_spectral_dim[4]['freq_hint'].append(a_ppm)

            has_assignments = has_multiple_assignments = False
            asis1 = asis2 = asis3 = asis4 = None

            if ass is not None:
                assignments = []
                if len(ass.split('-')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split('-'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(':')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(':'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(';')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(';'):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                elif len(ass.split(',')) == self.num_of_dim:
                    hint = None
                    for _ass in ass.split(','):
                        assignments.append(self.extractPeakAssignment(1, _ass, index, hint))
                        hint = assignments[-1] if assignments[-1] is not None else None
                else:
                    assignments = [None] * self.num_of_dim
                    _assignments = self.extractPeakAssignment(self.num_of_dim, ass, index)
                    if _assignments is not None and len(_assignments) == self.num_of_dim:
                        for idx in range(self.num_of_dim):
                            assignments[idx] = [_assignments[idx]]  # pylint: disable=unsubscriptable-object

                has_assignments, has_multiple_assignments, asis1, asis2, asis3, asis4 =\
                    self.checkAssignments4D(index, assignments)

            self.addAssignedPkRow4D(index, dstFunc, has_assignments, has_multiple_assignments,
                                    asis1, asis2, asis3, asis4,
                                    f'{ass} -> ', None if has_assignments and not has_multiple_assignments else ass)

        finally:
            self.numberSelection.clear()
            self.originalNumberSelection.clear()

    # Enter a parse tree produced by VnmrPKParser#number.
    def enterNumber(self, ctx: VnmrPKParser.NumberContext):
        if ctx.Float():
            value = str(ctx.Float())
            self.numberSelection.append(float(value))
            self.originalNumberSelection.append(value)

        elif ctx.Real():
            value = str(ctx.Real())
            self.numberSelection.append(float(value))
            self.originalNumberSelection.append(value)

        else:
            value = str(ctx.Integer())
            self.numberSelection.append(int(value))
            self.originalNumberSelection.append(value)

    # Exit a parse tree produced by VnmrPKParser#number.
    def exitNumber(self, ctx: VnmrPKParser.NumberContext):  # pylint: disable=unused-argument
        pass


# del VnmrPKParser



##
# XeasyPROTReader.py
#
# Update:
##
""" A collection of classes for parsing XEASY PROT files.
"""
__docformat__ = "restructuredtext en"
__author__ = "Masashi Yokochi"
__email__ = "yokochi@protein.osaka-u.ac.jp"
__license__ = "Apache License 2.0"
__version__ = "1.0.0"

import sys
import os

from antlr4 import InputStream, CommonTokenStream, ParseTreeWalker
from typing import IO, List, Tuple, Optional

try:
    from wwpdb.utils.nmr.mr.LexerErrorListener import LexerErrorListener
    from wwpdb.utils.nmr.mr.ParserErrorListener import ParserErrorListener
    from wwpdb.utils.nmr.pk.XeasyPROTLexer import XeasyPROTLexer
    from wwpdb.utils.nmr.pk.XeasyPROTParser import XeasyPROTParser
    from wwpdb.utils.nmr.pk.XeasyPROTParserListener import XeasyPROTParserListener
    from wwpdb.utils.nmr.mr.ParserListenerUtil import (coordAssemblyChecker,
                                                       MAX_ERROR_REPORT,
                                                       REPRESENTATIVE_MODEL_ID,
                                                       REPRESENTATIVE_ALT_ID)
    from wwpdb.utils.nmr.io.CifReader import CifReader
    from wwpdb.utils.nmr.ChemCompUtil import ChemCompUtil
    from wwpdb.utils.nmr.BMRBChemShiftStat import BMRBChemShiftStat
    from wwpdb.utils.nmr.nef.NEFTranslator import NEFTranslator
except ImportError:
    from nmr.mr.LexerErrorListener import LexerErrorListener
    from nmr.mr.ParserErrorListener import ParserErrorListener
    from nmr.pk.XeasyPROTLexer import XeasyPROTLexer
    from nmr.pk.XeasyPROTParser import XeasyPROTParser
    from nmr.pk.XeasyPROTParserListener import XeasyPROTParserListener
    from nmr.mr.ParserListenerUtil import (coordAssemblyChecker,
                                           MAX_ERROR_REPORT,
                                           REPRESENTATIVE_MODEL_ID,
                                           REPRESENTATIVE_ALT_ID)
    from nmr.io.CifReader import CifReader
    from nmr.ChemCompUtil import ChemCompUtil
    from nmr.BMRBChemShiftStat import BMRBChemShiftStat
    from nmr.nef.NEFTranslator import NEFTranslator


class XeasyPROTReader:
    """ Accessor methods for parsing XEASY PROT files.
    """

    def __init__(self, verbose: bool = True, log: IO = sys.stdout,
                 representativeModelId: int = REPRESENTATIVE_MODEL_ID,
                 representativeAltId: str = REPRESENTATIVE_ALT_ID,
                 mrAtomNameMapping: Optional[List[dict]] = None,
                 cR: Optional[CifReader] = None, caC: Optional[dict] = None, ccU: Optional[ChemCompUtil] = None,
                 csStat: Optional[BMRBChemShiftStat] = None, nefT: Optional[NEFTranslator] = None):
        self.__class_name__ = self.__class__.__name__
        self.__version__ = __version__

        self.__verbose = verbose
        self.__lfh = log
        self.__debug = False

        self.__maxLexerErrorReport = MAX_ERROR_REPORT
        self.__maxParserErrorReport = MAX_ERROR_REPORT

        self.__representativeModelId = representativeModelId
        self.__representativeAltId = representativeAltId
        self.__mrAtomNameMapping = mrAtomNameMapping

        # CCD accessing utility
        self.__ccU = ChemCompUtil(verbose, log) if ccU is None else ccU

        if cR is not None and caC is None:
            caC = coordAssemblyChecker(verbose, log, representativeModelId, representativeAltId,
                                       cR, self.__ccU, None, None, fullCheck=False)

        self.__cR = cR
        self.__caC = caC

        # BMRB chemical shift statistics
        self.__csStat = BMRBChemShiftStat(verbose, log, self.__ccU) if csStat is None else csStat

        # NEFTranslator
        self.__nefT = NEFTranslator(verbose, log, self.__ccU, self.__csStat) if nefT is None else nefT

    def setDebugMode(self, debug: bool):
        self.__debug = debug

    def setLexerMaxErrorReport(self, maxErrReport: int):
        self.__maxLexerErrorReport = maxErrReport

    def setParserMaxErrorReport(self, maxErrReport: int):
        self.__maxParserErrorReport = maxErrReport

    def parse(self, protFilePath: str, cifFilePath: Optional[str] = None, isFilePath: bool = True
              ) -> Tuple[Optional[XeasyPROTParserListener], Optional[ParserErrorListener], Optional[LexerErrorListener]]:
        """ Parse XEASY PROT file.
            @return: XeasyPROTParserListener for success or None otherwise, ParserErrorListener, LexerErrorListener.
        """

        ifh = None

        try:

            if isFilePath:
                protString = None

                if not os.access(protFilePath, os.R_OK):
                    if self.__verbose:
                        self.__lfh.write(f"+{self.__class_name__}.parse() {protFilePath} is not accessible.\n")
                    return None, None, None

                ifh = open(protFilePath, 'r')  # pylint: disable=consider-using-with
                input = InputStream(ifh.read())

            else:
                protFilePath, protString = None, protFilePath

                if protString is None or len(protString) == 0:
                    if self.__verbose:
                        self.__lfh.write(f"+{self.__class_name__}.parse() Empty string.\n")
                    return None, None, None

                input = InputStream(protString)

            if cifFilePath is not None:
                if not os.access(cifFilePath, os.R_OK):
                    if self.__verbose:
                        self.__lfh.write(f"+{self.__class_name__}.parse() {cifFilePath} is not accessible.\n")
                    return None, None, None

                if self.__cR is None:
                    self.__cR = CifReader(self.__verbose, self.__lfh)
                    if not self.__cR.parse(cifFilePath):
                        return None, None, None

            lexer = XeasyPROTLexer(input)
            lexer.removeErrorListeners()

            lexer_error_listener = LexerErrorListener(protFilePath, maxErrorReport=self.__maxLexerErrorReport)
            lexer.addErrorListener(lexer_error_listener)

            messageList = lexer_error_listener.getMessageList()

            if messageList is not None and self.__verbose:
                for description in messageList:
                    self.__lfh.write(f"[Syntax error] line {description['line_number']}:{description['column_position']} {description['message']}\n")
                    if 'input' in description:
                        self.__lfh.write(f"{description['input']}\n")
                        self.__lfh.write(f"{description['marker']}\n")

            stream = CommonTokenStream(lexer)
            parser = XeasyPROTParser(stream)
            # try with simpler/faster SLL prediction mode
            # parser._interp.predictionMode = PredictionMode.SLL  # pylint: disable=protected-access
            parser.removeErrorListeners()
            parser_error_listener = ParserErrorListener(protFilePath, maxErrorReport=self.__maxParserErrorReport)
            parser.addErrorListener(parser_error_listener)
            tree = parser.xeasy_prot()

            walker = ParseTreeWalker()
            listener = XeasyPROTParserListener(self.__verbose, self.__lfh,
                                               self.__representativeModelId,
                                               self.__representativeAltId,
                                               self.__mrAtomNameMapping,
                                               self.__cR, self.__caC,
                                               self.__ccU, self.__csStat, self.__nefT)
            walker.walk(listener, tree)

            messageList = parser_error_listener.getMessageList()

            if messageList is not None and self.__verbose:
                for description in messageList:
                    self.__lfh.write(f"[Syntax error] line {description['line_number']}:{description['column_position']} {description['message']}\n")
                    if 'input' in description:
                        self.__lfh.write(f"{description['input']}\n")
                        self.__lfh.write(f"{description['marker']}\n")
            elif messageList is None and cifFilePath is None:
                parser_error_listener = ParserErrorListener(protFilePath, maxErrorReport=self.__maxParserErrorReport)

            if self.__verbose and self.__debug:
                if listener.warningMessage is not None and len(listener.warningMessage) > 0:
                    self.__lfh.write(f"+{self.__class_name__}.parse() ++ Info  -\n" + '\n'.join(listener.warningMessage) + '\n')
                if isFilePath:
                    self.__lfh.write(f"+{self.__class_name__}.parse() ++ Info  - {listener.getContentSubtype()}\n")

            return listener, parser_error_listener, lexer_error_listener

        except IOError as e:
            if self.__verbose:
                self.__lfh.write(f"+{self.__class_name__}.parse() ++ Error  - {str(e)}\n")
            return None, None, None
        finally:
            if isFilePath and ifh is not None:
                ifh.close()


if __name__ == "__main__":
    reader = XeasyPROTReader(True)
    reader.setDebugMode(True)
    reader.parse('../../tests-nmr/mock-data-remediation/7a2d/all.prot',
                 '../../tests-nmr/mock-data-remediation/7a2d/7a2d.cif')



# Generated from XwinNmrPKParser.g4 by ANTLR 4.13.0
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO

def serializedATN():
    return [
        4,1,15,110,2,0,7,0,2,1,7,1,2,2,7,2,2,3,7,3,2,4,7,4,2,5,7,5,1,0,3,
        0,14,8,0,1,0,1,0,1,0,4,0,19,8,0,11,0,12,0,20,1,0,4,0,24,8,0,11,0,
        12,0,25,1,0,4,0,29,8,0,11,0,12,0,30,5,0,33,8,0,10,0,12,0,36,9,0,
        1,0,1,0,1,1,1,1,5,1,42,8,1,10,1,12,1,45,9,1,1,1,1,1,1,2,1,2,1,2,
        1,2,1,3,1,3,1,3,1,3,1,3,1,3,1,3,3,3,60,8,3,1,3,5,3,63,8,3,10,3,12,
        3,66,9,3,1,3,1,3,1,4,1,4,1,4,1,4,1,4,1,4,1,4,1,4,1,4,3,4,79,8,4,
        1,4,5,4,82,8,4,10,4,12,4,85,9,4,1,4,1,4,1,5,1,5,1,5,1,5,1,5,1,5,
        1,5,1,5,1,5,1,5,1,5,3,5,100,8,5,1,5,5,5,103,8,5,10,5,12,5,106,9,
        5,1,5,1,5,1,5,0,0,6,0,2,4,6,8,10,0,1,1,1,15,15,119,0,13,1,0,0,0,
        2,39,1,0,0,0,4,48,1,0,0,0,6,52,1,0,0,0,8,69,1,0,0,0,10,88,1,0,0,
        0,12,14,5,6,0,0,13,12,1,0,0,0,13,14,1,0,0,0,14,34,1,0,0,0,15,33,
        3,2,1,0,16,33,3,4,2,0,17,19,3,6,3,0,18,17,1,0,0,0,19,20,1,0,0,0,
        20,18,1,0,0,0,20,21,1,0,0,0,21,33,1,0,0,0,22,24,3,8,4,0,23,22,1,
        0,0,0,24,25,1,0,0,0,25,23,1,0,0,0,25,26,1,0,0,0,26,33,1,0,0,0,27,
        29,3,10,5,0,28,27,1,0,0,0,29,30,1,0,0,0,30,28,1,0,0,0,30,31,1,0,
        0,0,31,33,1,0,0,0,32,15,1,0,0,0,32,16,1,0,0,0,32,18,1,0,0,0,32,23,
        1,0,0,0,32,28,1,0,0,0,33,36,1,0,0,0,34,32,1,0,0,0,34,35,1,0,0,0,
        35,37,1,0,0,0,36,34,1,0,0,0,37,38,5,0,0,1,38,1,1,0,0,0,39,43,5,4,
        0,0,40,42,5,13,0,0,41,40,1,0,0,0,42,45,1,0,0,0,43,41,1,0,0,0,43,
        44,1,0,0,0,44,46,1,0,0,0,45,43,1,0,0,0,46,47,7,0,0,0,47,3,1,0,0,
        0,48,49,5,1,0,0,49,50,5,10,0,0,50,51,5,12,0,0,51,5,1,0,0,0,52,53,
        5,2,0,0,53,54,5,3,0,0,54,55,5,3,0,0,55,56,5,3,0,0,56,57,5,3,0,0,
        57,59,5,3,0,0,58,60,5,3,0,0,59,58,1,0,0,0,59,60,1,0,0,0,60,64,1,
        0,0,0,61,63,5,9,0,0,62,61,1,0,0,0,63,66,1,0,0,0,64,62,1,0,0,0,64,
        65,1,0,0,0,65,67,1,0,0,0,66,64,1,0,0,0,67,68,5,6,0,0,68,7,1,0,0,
        0,69,70,5,2,0,0,70,71,5,3,0,0,71,72,5,3,0,0,72,73,5,3,0,0,73,74,
        5,3,0,0,74,75,5,3,0,0,75,76,5,3,0,0,76,78,5,3,0,0,77,79,5,3,0,0,
        78,77,1,0,0,0,78,79,1,0,0,0,79,83,1,0,0,0,80,82,5,9,0,0,81,80,1,
        0,0,0,82,85,1,0,0,0,83,81,1,0,0,0,83,84,1,0,0,0,84,86,1,0,0,0,85,
        83,1,0,0,0,86,87,5,6,0,0,87,9,1,0,0,0,88,89,5,2,0,0,89,90,5,3,0,
        0,90,91,5,3,0,0,91,92,5,3,0,0,92,93,5,3,0,0,93,94,5,3,0,0,94,95,
        5,3,0,0,95,96,5,3,0,0,96,97,5,3,0,0,97,99,5,3,0,0,98,100,5,3,0,0,
        99,98,1,0,0,0,99,100,1,0,0,0,100,104,1,0,0,0,101,103,5,9,0,0,102,
        101,1,0,0,0,103,106,1,0,0,0,104,102,1,0,0,0,104,105,1,0,0,0,105,
        107,1,0,0,0,106,104,1,0,0,0,107,108,5,6,0,0,108,11,1,0,0,0,13,13,
        20,25,30,32,34,43,59,64,78,83,99,104
    ]

class XwinNmrPKParser ( Parser ):

    grammarFileName = "XwinNmrPKParser.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "'# PEAKLIST_DIMENSION'" ]

    symbolicNames = [ "<INVALID>", "Num_of_dim", "Integer", "Float", "COMMENT", 
                      "SPACE", "RETURN", "SECTION_COMMENT", "LINE_COMMENT", 
                      "Annotation", "Integer_ND", "SPACE_ND", "RETURN_ND", 
                      "Any_name", "SPACE_CM", "RETURN_CM" ]

    RULE_xwinnmr_pk = 0
    RULE_comment = 1
    RULE_dimension = 2
    RULE_peak_2d = 3
    RULE_peak_3d = 4
    RULE_peak_4d = 5

    ruleNames =  [ "xwinnmr_pk", "comment", "dimension", "peak_2d", "peak_3d", 
                   "peak_4d" ]

    EOF = Token.EOF
    Num_of_dim=1
    Integer=2
    Float=3
    COMMENT=4
    SPACE=5
    RETURN=6
    SECTION_COMMENT=7
    LINE_COMMENT=8
    Annotation=9
    Integer_ND=10
    SPACE_ND=11
    RETURN_ND=12
    Any_name=13
    SPACE_CM=14
    RETURN_CM=15

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.13.0")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class Xwinnmr_pkContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EOF(self):
            return self.getToken(XwinNmrPKParser.EOF, 0)

        def RETURN(self):
            return self.getToken(XwinNmrPKParser.RETURN, 0)

        def comment(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XwinNmrPKParser.CommentContext)
            else:
                return self.getTypedRuleContext(XwinNmrPKParser.CommentContext,i)


        def dimension(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XwinNmrPKParser.DimensionContext)
            else:
                return self.getTypedRuleContext(XwinNmrPKParser.DimensionContext,i)


        def peak_2d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XwinNmrPKParser.Peak_2dContext)
            else:
                return self.getTypedRuleContext(XwinNmrPKParser.Peak_2dContext,i)


        def peak_3d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XwinNmrPKParser.Peak_3dContext)
            else:
                return self.getTypedRuleContext(XwinNmrPKParser.Peak_3dContext,i)


        def peak_4d(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(XwinNmrPKParser.Peak_4dContext)
            else:
                return self.getTypedRuleContext(XwinNmrPKParser.Peak_4dContext,i)


        def getRuleIndex(self):
            return XwinNmrPKParser.RULE_xwinnmr_pk

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterXwinnmr_pk" ):
                listener.enterXwinnmr_pk(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitXwinnmr_pk" ):
                listener.exitXwinnmr_pk(self)




    def xwinnmr_pk(self):

        localctx = XwinNmrPKParser.Xwinnmr_pkContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_xwinnmr_pk)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 13
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==6:
                self.state = 12
                self.match(XwinNmrPKParser.RETURN)


            self.state = 34
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & 22) != 0):
                self.state = 32
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,4,self._ctx)
                if la_ == 1:
                    self.state = 15
                    self.comment()
                    pass

                elif la_ == 2:
                    self.state = 16
                    self.dimension()
                    pass

                elif la_ == 3:
                    self.state = 18 
                    self._errHandler.sync(self)
                    _alt = 1
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt == 1:
                            self.state = 17
                            self.peak_2d()

                        else:
                            raise NoViableAltException(self)
                        self.state = 20 
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,1,self._ctx)

                    pass

                elif la_ == 4:
                    self.state = 23 
                    self._errHandler.sync(self)
                    _alt = 1
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt == 1:
                            self.state = 22
                            self.peak_3d()

                        else:
                            raise NoViableAltException(self)
                        self.state = 25 
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,2,self._ctx)

                    pass

                elif la_ == 5:
                    self.state = 28 
                    self._errHandler.sync(self)
                    _alt = 1
                    while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                        if _alt == 1:
                            self.state = 27
                            self.peak_4d()

                        else:
                            raise NoViableAltException(self)
                        self.state = 30 
                        self._errHandler.sync(self)
                        _alt = self._interp.adaptivePredict(self._input,3,self._ctx)

                    pass


                self.state = 36
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 37
            self.match(XwinNmrPKParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class CommentContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def COMMENT(self):
            return self.getToken(XwinNmrPKParser.COMMENT, 0)

        def RETURN_CM(self):
            return self.getToken(XwinNmrPKParser.RETURN_CM, 0)

        def EOF(self):
            return self.getToken(XwinNmrPKParser.EOF, 0)

        def Any_name(self, i:int=None):
            if i is None:
                return self.getTokens(XwinNmrPKParser.Any_name)
            else:
                return self.getToken(XwinNmrPKParser.Any_name, i)

        def getRuleIndex(self):
            return XwinNmrPKParser.RULE_comment

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterComment" ):
                listener.enterComment(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitComment" ):
                listener.exitComment(self)




    def comment(self):

        localctx = XwinNmrPKParser.CommentContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_comment)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 39
            self.match(XwinNmrPKParser.COMMENT)
            self.state = 43
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==13:
                self.state = 40
                self.match(XwinNmrPKParser.Any_name)
                self.state = 45
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 46
            _la = self._input.LA(1)
            if not(_la==-1 or _la==15):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class DimensionContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Num_of_dim(self):
            return self.getToken(XwinNmrPKParser.Num_of_dim, 0)

        def Integer_ND(self):
            return self.getToken(XwinNmrPKParser.Integer_ND, 0)

        def RETURN_ND(self):
            return self.getToken(XwinNmrPKParser.RETURN_ND, 0)

        def getRuleIndex(self):
            return XwinNmrPKParser.RULE_dimension

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDimension" ):
                listener.enterDimension(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDimension" ):
                listener.exitDimension(self)




    def dimension(self):

        localctx = XwinNmrPKParser.DimensionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_dimension)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 48
            self.match(XwinNmrPKParser.Num_of_dim)
            self.state = 49
            self.match(XwinNmrPKParser.Integer_ND)
            self.state = 50
            self.match(XwinNmrPKParser.RETURN_ND)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_2dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self):
            return self.getToken(XwinNmrPKParser.Integer, 0)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(XwinNmrPKParser.Float)
            else:
                return self.getToken(XwinNmrPKParser.Float, i)

        def RETURN(self):
            return self.getToken(XwinNmrPKParser.RETURN, 0)

        def Annotation(self, i:int=None):
            if i is None:
                return self.getTokens(XwinNmrPKParser.Annotation)
            else:
                return self.getToken(XwinNmrPKParser.Annotation, i)

        def getRuleIndex(self):
            return XwinNmrPKParser.RULE_peak_2d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_2d" ):
                listener.enterPeak_2d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_2d" ):
                listener.exitPeak_2d(self)




    def peak_2d(self):

        localctx = XwinNmrPKParser.Peak_2dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_peak_2d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 52
            self.match(XwinNmrPKParser.Integer)
            self.state = 53
            self.match(XwinNmrPKParser.Float)
            self.state = 54
            self.match(XwinNmrPKParser.Float)
            self.state = 55
            self.match(XwinNmrPKParser.Float)
            self.state = 56
            self.match(XwinNmrPKParser.Float)
            self.state = 57
            self.match(XwinNmrPKParser.Float)
            self.state = 59
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==3:
                self.state = 58
                self.match(XwinNmrPKParser.Float)


            self.state = 64
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==9:
                self.state = 61
                self.match(XwinNmrPKParser.Annotation)
                self.state = 66
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 67
            self.match(XwinNmrPKParser.RETURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_3dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self):
            return self.getToken(XwinNmrPKParser.Integer, 0)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(XwinNmrPKParser.Float)
            else:
                return self.getToken(XwinNmrPKParser.Float, i)

        def RETURN(self):
            return self.getToken(XwinNmrPKParser.RETURN, 0)

        def Annotation(self, i:int=None):
            if i is None:
                return self.getTokens(XwinNmrPKParser.Annotation)
            else:
                return self.getToken(XwinNmrPKParser.Annotation, i)

        def getRuleIndex(self):
            return XwinNmrPKParser.RULE_peak_3d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_3d" ):
                listener.enterPeak_3d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_3d" ):
                listener.exitPeak_3d(self)




    def peak_3d(self):

        localctx = XwinNmrPKParser.Peak_3dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_peak_3d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 69
            self.match(XwinNmrPKParser.Integer)
            self.state = 70
            self.match(XwinNmrPKParser.Float)
            self.state = 71
            self.match(XwinNmrPKParser.Float)
            self.state = 72
            self.match(XwinNmrPKParser.Float)
            self.state = 73
            self.match(XwinNmrPKParser.Float)
            self.state = 74
            self.match(XwinNmrPKParser.Float)
            self.state = 75
            self.match(XwinNmrPKParser.Float)
            self.state = 76
            self.match(XwinNmrPKParser.Float)
            self.state = 78
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==3:
                self.state = 77
                self.match(XwinNmrPKParser.Float)


            self.state = 83
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==9:
                self.state = 80
                self.match(XwinNmrPKParser.Annotation)
                self.state = 85
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 86
            self.match(XwinNmrPKParser.RETURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Peak_4dContext(ParserRuleContext):
        __slots__ = 'parser'

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Integer(self):
            return self.getToken(XwinNmrPKParser.Integer, 0)

        def Float(self, i:int=None):
            if i is None:
                return self.getTokens(XwinNmrPKParser.Float)
            else:
                return self.getToken(XwinNmrPKParser.Float, i)

        def RETURN(self):
            return self.getToken(XwinNmrPKParser.RETURN, 0)

        def Annotation(self, i:int=None):
            if i is None:
                return self.getTokens(XwinNmrPKParser.Annotation)
            else:
                return self.getToken(XwinNmrPKParser.Annotation, i)

        def getRuleIndex(self):
            return XwinNmrPKParser.RULE_peak_4d

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPeak_4d" ):
                listener.enterPeak_4d(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPeak_4d" ):
                listener.exitPeak_4d(self)




    def peak_4d(self):

        localctx = XwinNmrPKParser.Peak_4dContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_peak_4d)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 88
            self.match(XwinNmrPKParser.Integer)
            self.state = 89
            self.match(XwinNmrPKParser.Float)
            self.state = 90
            self.match(XwinNmrPKParser.Float)
            self.state = 91
            self.match(XwinNmrPKParser.Float)
            self.state = 92
            self.match(XwinNmrPKParser.Float)
            self.state = 93
            self.match(XwinNmrPKParser.Float)
            self.state = 94
            self.match(XwinNmrPKParser.Float)
            self.state = 95
            self.match(XwinNmrPKParser.Float)
            self.state = 96
            self.match(XwinNmrPKParser.Float)
            self.state = 97
            self.match(XwinNmrPKParser.Float)
            self.state = 99
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==3:
                self.state = 98
                self.match(XwinNmrPKParser.Float)


            self.state = 104
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==9:
                self.state = 101
                self.match(XwinNmrPKParser.Annotation)
                self.state = 106
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 107
            self.match(XwinNmrPKParser.RETURN)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx





